#====================================================================================
# common.pro (C) 2004-2005 Michael 'Mickey' Lauer <mickey@Vanille.de>
#====================================================================================
#
# Purpose: This file contains qmake scope rules of common usage
#
# Example: When building a qmake based application using libsdl,
#          use CONFIG+=sdl to add the proper include and library
#          paths and definitions to the resulting Makefile
#
#
# General problem: One has to decide between evaluation at makefile generation time
#                  and evaluation at makefile processing time.
#                  The following example illustrates the difference:
#
# Evaluation at makefile processing time:
#
# sdl {
#     QMAKE_CFLAGS    += `sdl-config --cflags`
#     LIBS            += `sdl-config --libs`
# }
#
# Evalutation at makefile generation time:
#
# sdl {
#     QMAKE_CFLAGS    += $$system( sdl-config --cflags )
#     LIBS            += `sdl-config --libs`
# }
#
# For now I use version 2 which is a bit faster
# See the fine qmake manual for more details
#
#

#=============================================================================
# pthread scope for multithreaded applications
#

pthread {
    LIBS += -lpthread
}

#=============================================================================
# sdl scopes for applications using the Simple Direct Media Layer (SDL)
# http://www.libsdl.org
#

sdl {
    QMAKE_CXXFLAGS  += $$system( sdl-config --cflags )
    DEFINES         += USE_SDL QTOPIA
    LIBS            += $$system( sdl-config --libs )
}

sdl-mixer {
    LIBS            += -lSDL_mixer
}

sdl-image {
    LIBS            += -lSDL_image
}

sdl-font {
    LIBS            += -lSDL_ttf
}

#=============================================================================
# opie scopes for applications using the Open Palmtop Integrated Environment
# http://opie.handhelds.org
#

opie {
    DEFINES         += QWS
}

opiecore {
    LIBS            += -lopiecore2
}

opieui {
    CONFIG          += opiecore
    LIBS            += -lopieui2
}

opienet {
    CONFIG          += opiecore
    LIBS            += -lopienet2
}

opiepim {
    CONFIG          += opiecore
    LIBS            += -lopiepim2
}

opiedb {
    CONFIG          += opiecore
    LIBS            += -lopiedb2
}

opiemm {
    CONFIG          += opiecore
    LIBS            += -lopiemm2
}

opiesecurity {
    CONFIG          += opiecore
    LIBS            += -lopiesecurity
}


#=============================================================================
# scopes and rules for applications using the Enlightenment Foundation Libraries
# http://www.enlightenment.org
#

efl {
    QMAKE_CXXFLAGS  += $$system( pkg-config sigc++-2.0 --cflags )
    LIBS            += -lefl++

    debug {
            QMAKE_CXXFLAGS += -frtti -fexceptions
            QMAKE_CXXFLAGS -= -fno-rtti -fno-exceptions
            QMAKE_CXXFLAGS += -g3 -W -Wall
            QMAKE_CXXFLAGS += $$system( pkg-config libcwd --cflags )
            QMAKE_LIBS     += $$system( pkg-config libcwd --libs )
            message( "Configured for building against libcwd" )
    }

    !debug {
            QMAKE_CXXFLAGS -= -frtti -fexceptions
            QMAKE_CXXFLAGS += -fno-rtti -fno-exceptions
            QMAKE_CXXFLAGS += -O3
            message( "Configured for building a release version" )

    }
    message( "Configured for building against the EFL++" )
}

QMAKE_EDJE_CC  = edje_cc
edje_cc.output = $(EDIR)/data/edjes/${QMAKE_FILE_BASE}.edj
edje_cc.input = EDJES
edje_cc.commands = $${QMAKE_EDJE_CC} -o ${QMAKE_FILE_OUT} -v -id $(EDIR)/data/images -fd $(EDIR)/data/fonts ${QMAKE_FILE_IN}
edje_cc.CONFIG = no_link
edje_cc.variable_out = PRE_TARGETDEPS
edje_cc.name = EDJE
# QMAKE_EXTRA_COMPILERS += edje_cc
QMAKE_EXTRA_UNIX_COMPILERS += edje_cc

#=============================================================================
#
#
#=============================================================================
#
#
#=============================================================================
#
#
